@@ -22,7 +22,7 @@ main() {
 
     # TODO Update this to package the right artifacts
     cp target/$TARGET/release/rust-tuya-mqtt $stage/
-    cp config.json $stage/
+    cp *.json.sample $stage/
 
     cd $stage
     tar czf $src/$CRATE_NAME-$TRAVIS_TAG-$TARGET.tar.gz *
