@@ -12,6 +12,10 @@ use std::fmt::Display;
 use std::fs::File;
 use std::io::{BufReader, Write};
 use std::net::IpAddr;
+use std::str::FromStr;
+use std::sync::Arc;
+use std::thread;
+use std::time::Duration;
 
 mod socket;
 
@@ -19,10 +23,20 @@ mod socket;
 const SKIP: usize = 1;
 const RETRIES: usize = 3;
 
+#[derive(Deserialize, Serialize, Debug, Clone, Copy, PartialEq)]
 pub enum TuyaType {
+    #[serde(rename = "socket")]
     Socket,
 }
 
+impl FromStr for TuyaType {
+    type Err = serde_json::error::Error;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        serde_json::from_str(s)
+    }
+}
+
 type DeviceMap = HashMap<String, DeviceInfo>;
 
 #[derive(Deserialize, Serialize, Debug)]
@@ -51,22 +65,28 @@ fn default_port() -> u16 {
     1883_u16
 }
 
+fn default_devtype() -> TuyaType {
+    TuyaType::Socket
+}
+
 #[derive(Deserialize, Debug, Serialize, PartialEq, Clone)]
 struct DeviceInfo {
+    #[serde(default = "default_devtype")]
+    dev_type: TuyaType,
+    id: String,
+    ip: IpAddr,
+    key: String,
     name: String,
     version: String,
-    id: String,
-    key: String,
-    ip: IpAddr,
 }
 
 impl Display for DeviceInfo {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        write!(
-            f,
-            "Topic: Ip: {}, Id: {}, Key: {}, Version: {}",
-            self.ip, self.id, self.key, self.version
-        )
+        if std::env::var("TUYA_FULL_DISPLAY").map_or_else(|_| false, |_| true) {
+            write!(f, "{}", serde_json::to_string(self).unwrap())
+        } else {
+            write!(f, "{}", serde_json::to_string(&self.truncate()).unwrap())
+        }
     }
 }
 
@@ -74,11 +94,12 @@ impl Truncate for DeviceInfo {
     /// Take the last 5 characters and prefix them with "..."
     fn truncate(&self) -> DeviceInfo {
         DeviceInfo {
+            dev_type: self.dev_type,
+            id: String::from("...") + Self::truncate_str(&self.id),
+            ip: self.ip,
+            key: String::from("...") + Self::truncate_str(&self.key),
             name: self.name.clone(),
             version: self.version.clone(),
-            id: String::from("...") + Self::truncate_str(&self.id),
-            key: String::from("...") + Self::truncate_str(&self.key),
-            ip: self.ip,
         }
     }
 }
@@ -93,32 +114,38 @@ impl DeviceInfo {
             }
         }
         // Then try to parse the topic as a device
-        if content.len() > 5 {
+        if content.len() == 6 {
             return Ok(DeviceInfo {
                 name: "".to_string(),
+                dev_type: default_devtype(),
                 version: content[1].to_string(),
                 id: content[2].to_string(),
                 key: content[3].to_string(),
                 ip: content[4].parse()?,
             });
+        } else if content.len() > 7 {
+            return Ok(DeviceInfo {
+                name: "".to_string(),
+                dev_type: content[1].parse()?,
+                version: content[2].to_string(),
+                id: content[3].to_string(),
+                key: content[4].to_string(),
+                ip: content[5].parse()?,
+            });
         }
         Err(anyhow!("Not a valid Topic"))
     }
 }
 
-fn handle_publish(publish: Publish, devices: &DeviceMap, full_display: bool) -> Result<()> {
-    let topic: DeviceInfo = DeviceInfo::from_str_and_devices(&publish.topic, devices)
-        .context(format!("Trying to parse {}", &publish.topic))?;
+fn handle_publish(publish: Publish, devices: &DeviceMap) -> Result<()> {
+    let dev_info: DeviceInfo = DeviceInfo::from_str_and_devices(&publish.topic, devices)
+        .context(format!("Could not parse {}", &publish.topic))?;
 
-    if full_display {
-        debug!("{}", topic);
-    } else {
-        debug!("{}", topic.truncate());
-    }
+    debug!("{}", dev_info);
     let mqtt_state =
         std::str::from_utf8(&publish.payload).context("Mqtt payload is not valid utf8")?;
-    let tuya_payload = payload(&topic.id, TuyaType::Socket, &mqtt_state);
-    let tuya_device = TuyaDevice::create(&topic.version, Some(&topic.key), topic.ip)
+    let tuya_payload = payload(&dev_info.id, TuyaType::Socket, &mqtt_state);
+    let tuya_device = TuyaDevice::create(&dev_info.version, Some(&dev_info.key), dev_info.ip)
         .context("Could not create TuyaDevice")?;
     let pkid = publish.pkid as u32;
     set(&tuya_device, pkid, tuya_payload)
@@ -147,14 +174,16 @@ fn print_warnings_on_failure(pkid: u32, res: &TuyaResult<()>) {
 // due to bad behaving devices and resend the command until it get a valid response.
 fn set(dev: &TuyaDevice, pkid: u32, payload: Payload) -> Result<()> {
     use retry::{delay::Exponential, retry, Error};
-    match retry(
-        Exponential::from_millis(10).skip(SKIP).take(RETRIES),
-        || {
-            let r = dev.set(payload.clone(), pkid);
-            print_warnings_on_failure(pkid, &r);
-            r
-        },
-    ) {
+
+    let delay = Exponential::from_millis(10).skip(SKIP).take(RETRIES);
+
+    let try_set_payload = || {
+        let r = dev.set(payload.clone(), pkid);
+        print_warnings_on_failure(pkid, &r);
+        r
+    };
+
+    match retry(delay, try_set_payload) {
         Ok(()) => Ok(()),
         Err(e) => match e {
             Error::Operation {
@@ -167,18 +196,19 @@ fn set(dev: &TuyaDevice, pkid: u32, payload: Payload) -> Result<()> {
     }
 }
 
-fn handle_notification(event: Event, devices: &DeviceMap, full_display: bool) -> Result<()> {
+fn handle_notification(event: Event, devices: &DeviceMap) -> Result<()> {
     match event {
         Event::Incoming(packet) => match packet {
-            Packet::Publish(publish) => handle_publish(publish, devices, full_display)
-                .context("Handling publish notification."),
+            Packet::Publish(publish) => {
+                handle_publish(publish, devices).context("Handling publish notification.")
+            }
             _ => {
                 trace!("Unhandled incoming packet {:?}", packet);
                 Ok(())
             }
         },
         Event::Outgoing(packet) => {
-            trace!("Unhandled incoming packet {:?}", packet);
+            trace!("Unhandled outgoing packet {:?}", packet);
             Ok(())
         }
     }
@@ -205,34 +235,43 @@ fn initialize_logger() {
     }
 }
 
+fn read_devices(file: File) -> Result<Arc<DeviceMap>> {
+    let mut map: DeviceMap = HashMap::new();
+    info!("Reading devices from file");
+    let file_reader = BufReader::new(file);
+    let devices: Vec<DeviceInfo> = serde_json::from_reader(file_reader)?;
+    for device in devices.iter() {
+        map.insert(device.name.clone(), device.clone());
+    }
+    debug!("Read Devices:");
+    devices.iter().for_each(|dev| debug!("{}", dev));
+    Ok(Arc::new(map))
+}
+
 fn main() -> anyhow::Result<()> {
     initialize_logger();
-    let full_display = std::env::var("TUYA_FULL_DISPLAY").map_or_else(|_| false, |_| true);
     info!(
         "Full display {} - declare env variable TUYA_FULL_DISPLAY to set",
-        full_display
+        std::env::var("TUYA_FULL_DISPLAY").map_or_else(|_| false, |_| true)
     );
     info!("Reading config file");
-    let config: Config = serde_json::from_reader(BufReader::new(File::open("config.json")?))?;
+    let config: Config = serde_json::from_reader(BufReader::new(
+        File::open("config.json").context("No config.json file found in curdir")?,
+    ))
+    .context("Badly formatted config.json")?;
     debug!("Read {:#?}", config);
 
-    // Read the devices.json configuration file if it exist
-    let mut device_map: DeviceMap = HashMap::new();
-    if let Ok(file) = File::open("devices.json") {
-        info!("Reading devices from file");
-        let file_reader = BufReader::new(file);
-        let devices: Vec<DeviceInfo> = serde_json::from_reader(file_reader)?;
-        for device in devices.iter() {
-            device_map.insert(device.name.clone(), device.clone());
-        }
-        debug!("Read Devices {:#?}", devices);
-    }
+    // Read the devices.json configuration file if it exist, set empty device map otherwise
+    let device_map = File::open("devices.json")
+        .map_or(Ok(Arc::new(DeviceMap::new())), read_devices)
+        .context("Badly formatted devices.json")?;
 
     let mut options = MqttOptions::new(config.mqtt_id, config.host, config.port);
-    options.set_keep_alive(10);
+    options.set_keep_alive(Duration::from_secs(10));
     if !config.mqtt_user.is_empty() {
         options.set_credentials(config.mqtt_user, config.mqtt_pass);
     }
+    options.set_clean_session(false);
 
     let (mut client, mut connection) = Client::new(options, 10);
     let mut client2 = client.clone();
@@ -242,21 +281,23 @@ fn main() -> anyhow::Result<()> {
     })?;
     client.subscribe(
         format!("{}#", config.topic),
-        qos(config.qos).map_err(Error::msg)?,
+        qos(config.qos)
+            .map_err(Error::msg)
+            .context("Could not set MQTT QoS")?,
     )?;
     for n in connection.iter() {
-        // Give each thread a clone of the devices
         match n {
             Ok(n) => {
-                let devs = device_map.clone();
-                std::thread::spawn(move || match handle_notification(n, &devs, full_display) {
+                // Give each thread an Arc of the devices
+                let map = device_map.clone();
+                thread::spawn(move || match handle_notification(n, &map) {
                     Ok(_) => (),
                     Err(e) => error!("{}", e),
                 });
             }
             Err(e) => {
                 error!("{}", e);
-                std::thread::sleep(std::time::Duration::from_secs(5));
+                thread::sleep(Duration::from_secs(5));
             }
         };
     }
@@ -281,6 +322,7 @@ mod tests {
         assert_eq!(
             topic,
             DeviceInfo {
+                dev_type: TuyaType::Socket,
                 name: "".to_string(),
                 version: "ver3.3".to_string(),
                 id: "545c7250ecf8bc58a8fd".to_string(),
